package org.neuroph.contrib.tileclassification.screentools;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.Arrays;

/**
 * The intention of this class is to allow you to pay up front (at construction) 
 * the compute cost of converting the RGB values in a BufferedImage into a derived form.
 * 
 * The major benefit of using this class is a single loop that grabs all 3 color channels
 * (red, green, and blue) from each (x,y) coordinate, as opposed to looping through each
 * color channel (red, green, or blue) when you need it.
 * 
 * If you only need a single color from the channels (red, green, or blue) then
 * using this class may be more expensive than a custom solution.
 * 
 * In the event that it needs to be parsed, the flattened rgb values array contains 
 * all the red first, followed by all the green, followed by all the blue values.  
 * The flattened array size should be divisible by 3. 
 * 
 * @author Jon Tait
 *
 */
public class FractionRgbData
{
	private int width;
	private int height;
	
	protected double[][] redValues;
	protected double[][] greenValues;
	protected double[][] blueValues;
	protected double[] flattenedRgbValues;

	public FractionRgbData(BufferedImage img)
	{
		width = img.getWidth();
		height = img.getHeight();
		
		redValues = new double[height][width];
		greenValues = new double[height][width];
		blueValues = new double[height][width];
		flattenedRgbValues = new double[width * height * 3];
		
		populateRGBArrays(img);
	}

	protected void populateRGBArrays(BufferedImage img)
	{
		int width = getWidth();
		int height = getHeight();
		
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				Color color = new Color(img.getRGB(x, y));

				double red = ((double) color.getRed()) / 256d;
				redValues[y][x] = red;
				flattenedRgbValues[(y * width + x)] = red;

				double green = ((double) color.getGreen()) / 256d;
				greenValues[y][x] = green;
				flattenedRgbValues[(width * height + y * width + x)] = green;

				double blue = ((double) color.getBlue()) / 256d;
				blueValues[y][x] = blue;
				flattenedRgbValues[(2 * width * height + y * width + x)] = blue;
			}
		}
	}

	public int getWidth()
	{
		return width;
	}

	public int getHeight()
	{
		return height;
	}

	/**
	 * 
	 * @return 2d array in the form: [row][column]
	 */
	public double[][] getRedValues()
	{
		return redValues;
	}

	/**
	 * 
	 * @return 2d array in the form: [row][column]
	 */
	public double[][] getGreenValues()
	{
		return greenValues;
	}

	/**
	 * 
	 * @return 2d array in the form: [row][column]
	 */
	public double[][] getBlueValues()
	{
		return blueValues;
	}

	/**
	 * 
	 * @return All the red rows, followed by all the green rows, 
	 * followed by all the blue rows.
	 */
	public double[] getFlattenedRgbValues()
	{
		return flattenedRgbValues;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if(obj == null || !(obj instanceof FractionRgbData)) {
			return false;
		}
		FractionRgbData other = (FractionRgbData) obj;
		return Arrays.equals(flattenedRgbValues, other.getFlattenedRgbValues());
	}
	
	@Override
	public int hashCode()
	{
		return Arrays.hashCode(flattenedRgbValues);
	}
	
	@Override
	public String toString()
	{
		return Arrays.toString(flattenedRgbValues);
	}
}
